using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Android.App;
using Android.Content;
using Android.OS;
using Android.Runtime;
using Android.Views;
using Android.Widget;

using native = Com.Syncfusion.Gauges.SfLinearGauge;
using Syncfusion.SfGauge.XForms;

namespace Syncfusion.SfGauge.XForms.Droid
{
    public class LinearScaleMapping
    {

        #region Properties

		internal native.SfLinearGauge gauge { get; set; }
		native.LinearScale newScale;
        LinearPointerMapping pointerMapping;
        LinearTicksMapping ticksMapping;
        LinearRangeMapping rangeMapping;
		native.BarPointer nativeRangePointer;
		native.SymbolPointer nativeNeedlePointer;
        #endregion

        #region Methods

		public native.LinearScale GetScale(LinearScale formScale)
        {
			ticksMapping = new LinearTicksMapping();
			pointerMapping = new LinearPointerMapping();
			newScale = new native.LinearScale();
			nativeRangePointer = new native.BarPointer();
			nativeNeedlePointer = new native.SymbolPointer();
			rangeMapping = new LinearRangeMapping();
            
            
            if (formScale is LinearScale)
            {
				List<string> properties = SfLinearGaugeRenderer.GetPropertiesChanged(typeof(LinearScale), formScale);
                foreach (var propertyName in properties)
                {
					OnScalePropertiesChanged(propertyName, (LinearScale)formScale, (native.LinearScale)newScale);
                }
            }
           
            return newScale;
        }

		public void OnScalePropertiesChanged(string propertyName, LinearScale formScale, native.LinearScale nativeScale)
        {
			if (propertyName == LinearScale.RangesProperty.PropertyName) 
			{
				if (formScale.Ranges != null) {
					foreach (LinearRange formRange in formScale.Ranges) {
						formRange.BindingContext = formScale.BindingContext;
						var newRange = rangeMapping.GetRange (formRange);
						nativeScale.Ranges.Add (newRange);
                    
					}
				}
			}
			 if (propertyName == LinearScale.PointersProperty.PropertyName)
            {
                if (formScale.Pointers != null)
                {
                    foreach (LinearPointer FormPointer in formScale.Pointers)
                    {
                        FormPointer.BindingContext = formScale.BindingContext;
                        pointerMapping.gauge = this.gauge;
						pointerMapping.scalebar = formScale.ScaleBarSize;
                        var newPointer = pointerMapping.GetPointer(FormPointer);
                        FormPointer.PropertyChanged += FormPointer_PropertyChanged;
							if (newPointer is native.SymbolPointer)
								nativeScale.Pointers.Add((newPointer as native.SymbolPointer));
                        	else
								nativeScale.Pointers.Add((newPointer as native.BarPointer));
                    }
                }
            }
				 if (propertyName == LinearScale.MinimumValueProperty.PropertyName)
            {
					nativeScale.Minimum = formScale.MinimumValue;
            }
				else if (propertyName ==LinearScale.MaximumValueProperty.PropertyName)
            {
					nativeScale.Maximum = formScale.MaximumValue;
            }
            
            if (propertyName ==LinearScale.IntervalProperty.PropertyName)
            {
                nativeScale.Interval = formScale.Interval;
            }
           if (propertyName ==LinearScale.LabelPrefixProperty.PropertyName)
            {
                nativeScale.LabelPrefix = formScale.LabelPrefix;
            }
            if (propertyName ==LinearScale.LabelPostfixProperty.PropertyName)
            {
                nativeScale.LabelPostfix = formScale.LabelPostfix;
            }
			else if (propertyName ==LinearScale.ScaleBarColorProperty.PropertyName)
            {
				nativeScale.ScaleBarColor = new global::Android.Graphics.Color() { A = (byte)(formScale.ScaleBarColor.A * 255), R = (byte)(formScale.ScaleBarColor.R * 255), G = (byte)(formScale.ScaleBarColor.G * 255), B = (byte)(formScale.ScaleBarColor.B * 255) };
            }
			else if (propertyName ==LinearScale.ScaleBarLengthProperty.PropertyName)
            {
				nativeScale.ScaleBarLength = formScale.ScaleBarLength;
            }
			else if (propertyName ==LinearScale.ScaleBarSizeProperty.PropertyName)
            {
               nativeScale.ScaleBarSize = formScale.ScaleBarSize*3;
            }
             if (propertyName ==LinearScale.LabelColorProperty.PropertyName)
            {
                nativeScale.LabelColor = new global::Android.Graphics.Color() { A = (byte)(formScale.LabelColor.A * 255), R = (byte)(formScale.LabelColor.R * 255), G = (byte)(formScale.LabelColor.G * 255), B = (byte)(formScale.LabelColor.B * 255) };
            }
            if (propertyName ==LinearScale.MinorTicksPerIntervalProperty.PropertyName)
            {
                nativeScale.MinorTicksPerInterval = formScale.MinorTicksPerInterval;
            }
            if (propertyName ==LinearScale.MajorTickSettingsProperty.PropertyName)
            {
                formScale.MajorTickSettings.BindingContext = formScale.BindingContext;
               var newticks = ticksMapping.GetTicks(formScale.MajorTickSettings);
               nativeScale.MajorTickSettings = newticks;
            }
             if (propertyName ==LinearScale.MinorTickSettingsProperty.PropertyName)
            {
                formScale.MinorTickSettings.BindingContext = formScale.BindingContext;
                var newticks = ticksMapping.GetTicks(formScale.MinorTickSettings);
                nativeScale.MinorTickSettings = newticks;
            }
            if (propertyName ==LinearScale.ShowLabelsProperty.PropertyName)
            {
                nativeScale.ShowLabels = formScale.ShowLabels;
            }
//            else if (propertyName ==LinearScale.ShowTicksProperty.PropertyName)
//            {
//                nativeScale.ShowTicks = formScale.ShowTicks;
//            }
//            else if (propertyName ==LinearScale.ShowRimProperty.PropertyName)
//            {
//                nativeScale.ShowRim = formScale.ShowRim;
//            }
//            else if (propertyName ==LinearScale.RadiusFactorProperty.PropertyName)
//            {
//                nativeScale.RadiusFactor = formScale.RadiusFactor;
//            }
//            else if (propertyName ==LinearScale.LabelOffsetProperty.PropertyName)
//            {
//                nativeScale.LabelsFactor = formScale.LabelOffset;
//            }
        
		}
        void FormPointer_PropertyChanged(object sender, System.ComponentModel.PropertyChangedEventArgs e)
        {
			if (sender is SymbolPointer)
            {
				List<string> properties = SfLinearGaugeRenderer.GetPropertiesChanged(typeof(SymbolPointer), sender);
                foreach (var propertyName in properties)
                {
					pointerMapping.OnPointerPropertiesChanged(propertyName, (SymbolPointer)sender, (native.SymbolPointer)nativeNeedlePointer);
                }
            }
            else
            {
				List<string> properties = SfLinearGaugeRenderer.GetPropertiesChanged(typeof(BarPointer), sender);
                foreach (var propertyName in properties)
                {
					pointerMapping.OnPointerPropertiesChanged(propertyName, (BarPointer)sender, (native.BarPointer)nativeRangePointer);
                }
            }
        }
        #endregion
    }
}